/*  PROGRAM NAME : ANSI2UTF

    AUTHOR       : Marc Engrie

    DESCRIPTION  : Converts a ANSI text file to a UTF-8 text file.

    INSTRUCTIONS : Run this program with no arguments for complete instructions.

    LANGUAGE     : Microsoft C  Version 8.00c
*/

//****************************************************************************
// REVISION HISTORY                                                          *
/*****************************************************************************
|   Date   | Rev. | Modifications                                            |
|----------------------------------------------------------------------------|
| 05/04/2005 | 1.00 | Initial Release                                        |
| 05/04/2005 | 1.01 | Correct some minor problems                            |
| 05/04/2005 | 1.02 | Correct some minor problems                            |
*****************************************************************************/
//****************************************************************************
//  Include files                                                            *
//****************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <direct.h>
#include <io.h>
#include <errno.h>

//****************************************************************************
//  Defines                                                                  *
//****************************************************************************

#define VERSION    "1.02"
#define MIN_ARG     2

// Return code defines
#define NO_ERRORS                 0
#define WRITE_ERROR              10

#define NOT_ENOUGH_ARGUMENTS    202
#define WRONG_ARGUMENTS         203

//****************************************************************************
//  Overall variables                                                        *
//****************************************************************************

// strings
char    *pCopyright = "%s  Version : %s  written by Marc Engrie\n";

// used for program information
char    szDrive[_MAX_DRIVE];
char    szDir[_MAX_DIR];
char    szFile[_MAX_FNAME];
char    szExt[_MAX_EXT];
char    szTmp[_MAX_DRIVE + _MAX_DIR + _MAX_FNAME + _MAX_EXT];

char    szInputFile[20];                // name of inputfile
char    *pOutputFile;                   // pointer to output filename
FILE    *hInputFile;                    // file handle
FILE    *hOutputFile;                   // file handle

//****************************************************************************
//  Functions used                                                           *
//****************************************************************************

void    Exit(int iReturnCode);
void    prtusage(char *pProgName);

//============================================================================
//  MAIN PROGRAM
//============================================================================

int main(int argc, char* argv[])
{
    char    *pcArgument;                    // argument pointer
    int     iArg;                           // argument index
    int     iChar;                          // actual character from file
    int     iActualPercent;                 // actual percentage done
    int     iPreviousPercent;               // previous percentage done
    long    lLoc;                           // location & length file
    long    lLen;                           // location & length file

    // get program name
    _splitpath(argv[0], szDrive, szDir, szFile, szExt);

    // who wrote this 'master'(??) piece
    //fprintf(stderr, pCopyright, strupr(szFile), VERSION);
    printf(pCopyright, strupr(szFile), VERSION);

    /*************************************************************************
    * no args - give help
    *************************************************************************/

    if(argc < MIN_ARG)
    {
        prtusage(szFile);
        return(NOT_ENOUGH_ARGUMENTS);
    }

    //***********************************************************************
    // init variables
    //***********************************************************************

    szInputFile[0] = '\0';

    //***********************************************************************
    // check command line
    //***********************************************************************

    // scan the arguments
    for(iArg = 1; iArg < argc; iArg++)
    {
        // convert to uppercase first
        pcArgument = strupr(argv[iArg]);

        // an option ?
        if(*pcArgument == '/')
        {
            prtusage(szFile);
            exit(1);
        }
        else
        {
            // get file to convert
            strcpy(szInputFile,pcArgument);
        }
    }

    //***********************************************************************
    // Open files
    //***********************************************************************

    // do we have a file name
    if(szInputFile[0])
    {
        // then try to use it
        if((hInputFile=fopen(szInputFile, "rb")) == NULL)
        {
            fprintf(stderr, "\a%s : %s\n", szInputFile, _strerror(""));
            exit(WRONG_ARGUMENTS);
        }
    }
    else
    {
        fprintf(stderr, "\nInput filename not specified\n");
        exit(WRONG_ARGUMENTS);
    }

    // get length of file
    lLen=filelength(fileno(hInputFile));

    // prepare for tmp file
    // get current directory
    getcwd(szDir, _MAX_DIR);
    // set up TMP=
    strcpy(szTmp, "TMP=");
    // complete with current dir
    strcat(szTmp, szDir);
    // setup environment
    putenv(szTmp);
    // create temp file name
    pOutputFile = tempnam(getcwd(szDir, _MAX_DIR), "A2U");

    // do we have a temp file
    if(pOutputFile != NULL)
    {
        // then try to use it
        if((hOutputFile=fopen(pOutputFile, "wb")) == NULL)
        {
            fprintf(stderr, "\a%s : %s\n", pOutputFile, _strerror(""));
            Exit(WRONG_ARGUMENTS);
        }
    }
    else
    {
        fprintf(stderr, "\nOutput filename %s has a problem\n", pOutputFile);
        Exit(WRONG_ARGUMENTS);
    }

    //***********************************************************************
    // main routine that changes LF to CRLF
    //***********************************************************************

    iActualPercent = 0;
    iPreviousPercent = 0;
    lLoc = 0;

    printf("%03i%% done of file: %s\r", iActualPercent, szInputFile);

    // write UTF-8 header
    if(fputc(0xEF, hOutputFile) == EOF)
    {
        Exit(WRITE_ERROR);
    }
    if(fputc(0xBB, hOutputFile) == EOF)
    {
        Exit(WRITE_ERROR);
    }
    if(fputc(0xBF, hOutputFile) == EOF)
    {
        Exit(WRITE_ERROR);
    }

    // while more characters to get
    while((iChar=fgetc(hInputFile)) != EOF)
    {
        // save position = savpos
        lLoc++;
        // how far are we in %
        iActualPercent = (int)((lLoc * 100) / lLen);
        // only display when changed
        if (iActualPercent > iPreviousPercent)
        {
            printf("%03i%%\r", iActualPercent);
            iPreviousPercent = iActualPercent;
        }

        // convert to UTF-8 if required
        if (iChar < 0x80)
        {
            if(fputc(iChar, hOutputFile) == EOF)
            {
                Exit(WRITE_ERROR);
            }
        }
        else
        {
            if(fputc(0xC0 | iChar >> 6, hOutputFile) == EOF)
            {
                Exit(WRITE_ERROR);
            }
            if(fputc(0x80 | iChar & 0x3F, hOutputFile) == EOF)
            {
                Exit(WRITE_ERROR);
            }
        }
    }; // loop until EOF

    //printf("\n");

    //***********************************************************************
    //   End of program
    //***********************************************************************

    fcloseall();
    remove(szInputFile);
    rename((const char *)pOutputFile, (const char *)szInputFile);
    free(pOutputFile);
    return(NO_ERRORS);
}

//****************************************************************************
//  Functions used                                                           *
//****************************************************************************

/******************************************************************************
*    prtusage
******************************************************************************/
void    prtusage(char *pProgName)
{
    printf("\n");
    printf("Usage: %s filename.ext [/?]\n\n",pProgName);
    printf("Where: filename.ext  ANSI filename to convert to UTF-8\n");
    printf("       /?            This help screen\n\n");
    printf("Remark: handles only 8.3 filenames\n");
    return;
}

/******************************************************************************
*    Exit
******************************************************************************/
void    Exit(int iReturnCode)
{
    fcloseall();                                            // close all files
    free(pOutputFile);
    // remove((const char *)pOutputFile);
    exit(iReturnCode);
}
